/**********************************************/
/* SciView data viewer for Hiew (c) FSC, 2007 */
/**********************************************/

#ifndef SCIVIEW_H
#define SCIVIEW_H

#include <windows.h>
#include "scintilla.h"

//
// Error codes
//
#define SCIVIEW_ERR_NOTIMPLEMENTEDYET   -2000   // this feature is not yet implemented
#define SCIVIEW_ERR_FOLDINGOVERFLOW     -2001   // too many SCIVIEW_STARTFOLDING commands
#define SCIVIEW_ERR_FOLDINGUNDERFLOW    -2002   // too many SCIVIEW_ENDFOLDING commands
#define SCIVIEW_ERR_NOSCIVIEWWINDOW     -2003   // an atempt to use folding while there is no Scintilla window
#define SCIVIEW_ERR_BADFOLDINGSEQUENCE  -2004   // line marked with SCIVIEW_STARTFOLDING command must be lesser than SCIVIEW_ENDFOLDING line
#define SCIVIEW_ERR_SINGLELINEFOLDING   -2005   // there is no sense to use folding for single line
#define SCIVIEW_ERR_DUPLICATEFOLDING    -2006   // a request to make a particular line to be a fold point was issued twice or more times

//
// SciView control messages
//
#define SCIVIEW_SHOW            0               // SCIVIEW_PARAM_SHOW
#define SCIVIEW_HIDE            1               // no parameters
#define SCIVIEW_GETINFO         2               // fills in SCIVIEW_PARAM_GETINFO
#define SCIVIEW_SETSTYLE        3               // IntValue
#define SCIVIEW_SETINDENT       4               // IntValue
#define SCIVIEW_INCINDENT       5               // no parameters
#define SCIVIEW_DECINDENT       6               // no parameters
#define SCIVIEW_ADDTEXT         7               // pText
#define SCIVIEW_SETUPSTYLE      8               // SCIVIEW_PARAM_SETUPSTYLE
#define SCIVIEW_CLEAR           9               // no parameters
#define SCIVIEW_ADDTEXTS        10              // PSCIVIEW_PARAM_ADDTEXTS
#define SCIVIEW_SETUPSTYLES     11              // PSCIVIEW_PARAM_SETUPSTYLES
#define SCIVIEW_GETURL          12              // PSCIVIEW_PARAM_GETURL
#define SCIVIEW_VERSIONMAJOR    13              // no parameters
#define SCIVIEW_VERSIONMINOR    14              // no parameters
#define SCIVIEW_STARTFOLDING    15              // no parameters
#define SCIVIEW_ENDFOLDING      16              // no parameters
#define SCIVIEW_HIDEFOLDINGS    17              // IntValue: 0 - do nothing with new foldings, !0 - autohide foldings
#define SCIVIEW_ADDTEXTLEN      18              // PSCIVIEW_PARAM_ADDTEXTLEN

//
// parameters for all available commands
//
typedef struct _SCIVIEW_PARAM_SHOW
{
    DLGPROC DialogProc;                     // pointer to user supplied dialog procedure or 0
} SCIVIEW_PARAM_SHOW, *PSCIVIEW_PARAM_SHOW;

typedef struct _SCIVIEW_PARAM_GETINFO
{
    HWND hHiewWnd;                          // handle of Hiew console window
    HWND hDialogWnd;                        // handle of dialog
    HWND hSciViewWnd;                       // handle of SciView control
    UINT SciViewId;                         // dialog ID of SciView control
    LPCTSTR SciViewClassName;               // pointer to class name of SciView control
    SciFnDirect DirectFn;                   // address of Scintilla DirectFunction
    sptr_t DirectPtr;                       // Scintilla DirectPtr
} SCIVIEW_PARAM_GETINFO, *PSCIVIEW_PARAM_GETINFO;

typedef struct _SCIVIEW_PARAM_SETUPSTYLE
{
    int Style;                              // number of style to setup
    UINT Flags;                             // which fields are valid
    LPCTSTR pFontName;                      // font name
    int FontSize;                           // font size
    BYTE IsBold;                            // font is bold if this is !0
    BYTE IsItalic;                          // font is italic if this is !0
    BYTE IsUnderline;                       // font is underlined if this is !0
    BYTE FillEOL;                           // EOLs are filled with background color if this is !0
    BYTE HotSpot;                           // style is used for URLs etc.
    COLORREF TextColor;                     // text color
    COLORREF BackColor;                     // background color
} SCIVIEW_PARAM_SETUPSTYLE, *PSCIVIEW_PARAM_SETUPSTYLE;

#define SETUPSTYLE_FONTNAME     0x01
#define SETUPSTYLE_FONTSIZE     0x02
#define SETUPSTYLE_ISBOLD       0x04
#define SETUPSTYLE_ISITALIC     0x08
#define SETUPSTYLE_ISUNDERLINE  0x10
#define SETUPSTYLE_FILLEOL      0x20
#define SETUPSTYLE_HOTSPOT      0x40
#define SETUPSTYLE_TEXTCOLOR    0x80
#define SETUPSTYLE_BACKCOLOR    0x100
#define SETUPSTYLE_ALL          SETUPSTYLE_FONTNAME | SETUPSTYLE_FONTSIZE | SETUPSTYLE_ISBOLD | SETUPSTYLE_ISITALIC | SETUPSTYLE_ISUNDERLINE | SETUPSTYLE_FILLEOL | SETUPSTYLE_HOTSPOT | SETUPSTYLE_TEXTCOLOR | SETUPSTYLE_BACKCOLOR

typedef struct _SCIVIEW_PARAM_SETUPSTYLES
{
    PSCIVIEW_PARAM_SETUPSTYLE pStyles;
    UINT NumStyles;
} SCIVIEW_PARAM_SETUPSTYLES, *PSCIVIEW_PARAM_SETUPSTYLES;

typedef struct _STYLED_TEXT
{
    LPCTSTR pText;
    int Style;
} STYLED_TEXT, *PSTYLED_TEXT;

typedef struct _SCIVIEW_PARAM_ADDTEXTS
{
    PSTYLED_TEXT pTexts;                    // pointer to array of separately styled messages
    UINT NumTexts;                          // number of entries in the array
} SCIVIEW_PARAM_ADDTEXTS, *PSCIVIEW_PARAM_ADDTEXTS;

typedef struct _SCIVIEW_PARAM_GETURL
{
    int Pos;                                // position in SciView from which to retrieve equally-styled text
    LPSTR pOutText;                         // pointer to output buffer or 0 if user wants to know necessary buffer size
} SCIVIEW_PARAM_GETURL, *PSCIVIEW_PARAM_GETURL;

typedef struct _SCIVIEW_PARAM_ADDTEXTLEN
{
    LPCTSTR pText;
    int TextLen;
    int Style;
} SCIVIEW_PARAM_ADDTEXTLEN, *PSCIVIEW_PARAM_ADDTEXTLEN;

//
// HEM2HEMGATE main call structure
//
typedef struct _SCIVIEW_COMMAND
{
    UINT cbSize;                            // size of this structure
    UINT Command;                           // command code
    union
    {
        SCIVIEW_PARAM_SHOW Show;            // SCIVIEW_SHOW
        SCIVIEW_PARAM_GETINFO GetInfo;      // SCIVIEW_GETINFO
        int IntValue;                       // SCIVIEW_SETSTYLE, SCIVIEW_SETINDENT, SCIVIEW_HIDE, SCIVIEW_GETURL
        LPCTSTR pText;                      // SCIVIEW_ADDTEXT
        SCIVIEW_PARAM_SETUPSTYLE Style;     // SCIVIEW_SETUPSTYLE
        PSCIVIEW_PARAM_ADDTEXTS pTexts;     // SCIVIEW_ADDTEXTS
        PSCIVIEW_PARAM_SETUPSTYLES pStyles; // SCIVIEW_SETUPSTYLES
        SCIVIEW_PARAM_GETURL Url;           // SCIVIEW_GETURL
        SCIVIEW_PARAM_ADDTEXTLEN TextLen;   // SCIVIEW_ADDTEXTLEN
    };

} SCIVIEW_COMMAND, *PSCIVIEW_COMMAND;

#define SIZEOFARRAY(Arr)    sizeof(Arr)/sizeof(Arr[0])
#define DEFINETEXTS(Name, Msgs) SCIVIEW_PARAM_ADDTEXTS Name = {Msgs, SIZEOFARRAY(Msgs)}
#define DEFINESTYLES(Name, Styles) SCIVIEW_PARAM_SETUPSTYLES Name = {Styles, SIZEOFARRAY(Styles)}

#endif